/*
 * Created on 20.03.2004
 */
package points;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.WindowConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;



/**
 * JFrame which contains the PointPanel and a JList of SelectablePoints 
 */
public class PointFrameWithList extends JFrame {

    protected PointModel pointModel;
    protected JList pointsList;

    public PointFrameWithList(PointModel model) {
    	super("Points with JList"); 
    	setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE); 
		Container cp = this.getContentPane(); 
  	
    	// PointModel
		this.pointModel = model; 
		
		// PointPanel
		PointPanel panel = new PointPanel(this.pointModel); 
		cp.add(panel, BorderLayout.CENTER); 
        
        // JList
        pointsList = new JList(pointModel);
        
        pointsList.setCellRenderer(new PointCellRenderer()); 
        
        pointsList.setPreferredSize(new Dimension(160, 200)); 
        pointsList.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                Object[] selectedPoints = pointsList.getSelectedValues();
                markPointsSelected(selectedPoints);
                System.out.print("Selected : ");
                for (int i = 0; i < selectedPoints.length; i++) {
                    Point p = (Point)selectedPoints[i];
                    System.out.print(p.x + "/" + p.y + ",  ");
                }
                System.out.println();
            }
        });
        JScrollPane scrollPane = new JScrollPane(pointsList);
        scrollPane.setPreferredSize(new Dimension(160, 100));
        cp.add(scrollPane, BorderLayout.WEST);
        
        // create the buttons for the selection modes 
        JPanel buttonPanel = new JPanel();
        JButton singleSelButton = new JButton("Single");
        singleSelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pointsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            }
        });
        buttonPanel.add(singleSelButton);
        JButton singleIntervalButton = new JButton("Single Interval");
        singleIntervalButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pointsList.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
            }
        });
        buttonPanel.add(singleIntervalButton);
        JButton multipleIntervalButton = new JButton("Multiple Interval");
        multipleIntervalButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pointsList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
            }
        });
        buttonPanel.add(multipleIntervalButton);
        cp.add(buttonPanel, BorderLayout.SOUTH);
        
		// Menu
		JMenuBar bar = new JMenuBar(); 
		this.setJMenuBar(bar); 
		JMenu fileMenu = new JMenu("File"); 
		bar.add(fileMenu); 
		JMenuItem exit = new JMenuItem("Exit"); 
		fileMenu.add(exit); 
		exit.addActionListener((new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0); 
			}
		})); 
		JMenu editMenu = new JMenu("Edit"); 
		bar.add(editMenu); 
		JMenuItem del = new JMenuItem("Delete"); 
		editMenu.add(del); 
		del.addActionListener((new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				for (Point p: pointModel.getSelected())
					pointModel.remove(p); 
			}
		})); 		
    }

    /** Sets the selected flag of the points selected in the JList
     * @param selectedPoints
     */
    protected void markPointsSelected(Object[] selectedPoints) {
    	pointModel.clearSelected(); 
    	for (Object selected: selectedPoints) {
    		pointModel.setSelected((Point) selected); 
    	}
       this.getContentPane().repaint();
    }

    /** CellRenderer for the elements in the JList. 
     * Renders SelectablePoints by giving out the string Point, the coordinates and the selected state. 
     * Renders selected points in black and not selected states in light gray. 
   	 */
    class PointCellRenderer implements ListCellRenderer {
        public Component getListCellRendererComponent(
            final JList list,
            final Object value,
            final int index,
            final boolean isSelected,
            final boolean cellHasFocus) {
            	
            Point p = (Point)value;
            JLabel label = new JLabel(index + " Point(" + p.x + "/" + p.y + ")" + pointModel.isSelected(p)); 
            if (isSelected) {
            	label.setForeground(Color.RED);
            	label.setBackground(list.getSelectionBackground()); 
            } else {
            	label.setForeground(Color.GRAY); 
               	label.setBackground(list.getBackground()); 
            }
            
            return label;
        }
    }

    /** Main method creates the ListModel and the PointFrameWithList object 
     * @param args the command line arguments
     */
    public static void main(String args[]) {
    	
        PointModel points = new PointModel();
        
        PointFrameWithList frame = new PointFrameWithList(points);
        frame.setSize(600, 600); 
        frame.setVisible(true);
        
    }
    
}
